'''

 Filter to create a NEW image from the INPUT DRAWABLE
 having the look of a PLAIN KNITTED sweater
 ............................................................. 
 
 Release 1.0   initial
 
 Flow implemented to get the final result:      
 
 1. save current context
 2. prepare the new image, copying the source drawable into the new BG
 3. duplicate it creating a work image for quantization
 4. once properly quantized, bring back the layer into the output image  
 5. duplicate the quantized layer   
    apply Newsprint filter
 6. duplicate the Newsprint layer
    apply the Shift filter horizontally
 7. duplicate the Newsprint layer again
    apply the Shift filter vertically
 8. create a new layer for the knit pattern
    fill with the "MShine07.png" pattern
    desaturate
    increase brightness and contrast
    apply the Ripple filter
 9. raise the BG on top, GRAIN_MERGE
10. flatten the new image if requested
11. restore context
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
  
'''
#!/usr/bin/env python
from gimpfu import *
import decimal
import random
import time
import datetime

# Service routines   
def get_saturation (layer):

    image = pdb.gimp_item_get_image (layer)
    sat_image = pdb.plug_in_decompose_registered (image, layer, "Saturation", False )
    #decompose returns an array, in this case of 1 element
    sat_layer = pdb.gimp_image_get_active_layer (sat_image[0]) #element 0 is the only one returned
    saturation = pdb.gimp_histogram (sat_layer, HISTOGRAM_VALUE, 0, 255 )
    pdb.gimp_image_delete (sat_image[0])
    return(saturation[0])

def get_value(layer):

    image = pdb.gimp_item_get_image (layer)
    val_image = pdb.plug_in_decompose_registered (image, layer, "Value", False )
    #decompose returns an array, in this case of 1 element
    val_layer = pdb.gimp_image_get_active_layer (val_image[0]) #element 0 is the only one returned
    value = pdb.gimp_histogram (val_layer, HISTOGRAM_VALUE, 0, 255 )
    pdb.gimp_image_delete (val_image[0])
    return(value[0])

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#
def PlainKnittedLook (inImage, inDrawable, widthheight,
                      nrAreas, doFlatten,
                     ) :

    pdb.gimp_context_push()
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    width = inImage.width
    height = inImage.height
    type = inImage.base_type
    
    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # calculate scale factor for the new output image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = width+height
    # check/adjust size not smaller than 2000 as w+h
    if widthheight < 2000 :   # not too small, pls
        widthheight = 2000
    
    optiWidthHeight = widthheight       # size to work on to get best results
    realWidthHeight = width+height      # real size of the input image
    scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling
    
    if nrAreas == 0: # auto 
        nrAreas = int(optiWidthHeight/150)
        if nrAreas < 8:
            nrAreas = 8
        if nrAreas > 32:
            nrAreas = 32
            
    suffix = ", having a knitted look, on "+str(int(nrAreas))+" colour areas"

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)
# fix all required context settings 
    pdb.gimp_context_set_antialias(True)
    pdb.gimp_context_set_feather(False)
    pdb.gimp_context_set_feather_radius(0, 0)
    pdb.gimp_context_set_sample_merged(False) 
    pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE) 
    pdb.gimp_context_set_sample_threshold(0.00)
    pdb.gimp_context_set_sample_transparent(False)       # beware, if TRUE may change the area selected!
    pdb.gimp_context_set_interpolation(3)                # 1.0.1 INTERPOLATION_LANCZOS not understood by Gimp 2.9.5
    
    newwidth=int(width*scale)
    newheight=int(height*scale)
    
    # ==================================================================
    # NEW OUTPUT IMAGE
    # ------------------------------------------------------------------
    # 
    newImage = pdb.gimp_image_new (newwidth, newheight, type)                    
    newBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)     # background layer
    newBG.mode = NORMAL_MODE
    newBG.name = "newBG: Background Image Layer"
    newBG.opacity = 100.0 
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale(newBG, newwidth, newheight, False)
    pdb.gimp_layer_add_alpha (newBG)    #  RGBA
    # check/avoid too dark source images
    meanV = get_value (newBG)
    info_msg ("meanV = "+str(meanV))
    if meanV < 120:
        #pdb.gimp_brightness_contrast (newBG, -60, +30)
        pdb.gimp_curves_spline (newBG, HISTOGRAM_VALUE, 10, (0,0, 8,24, 32,64, 96,140, 255,255))
        pdb.gimp_hue_saturation (newBG, 0, 0, 0, +60)   # was +100
    # check/avoid too saturated source images
    satV = get_saturation (newBG)
    if satV > 111:
        pdb.gimp_hue_saturation (newBG, 0, 0, 0, -40)
    
    # =====================================================================
    # use work images to create the quantized layer
    # ---------------------------------------------------------------------
    #
    QUANT_IMAGE = pdb.gimp_image_duplicate (newImage)
    QUANT_DRAWABLE = QUANT_IMAGE.active_layer
    QUANT_DRAWABLE.name = "Quant BG Layer"
    
    # change image mode to indexed 
    
    pdb.gimp_image_convert_indexed (QUANT_IMAGE, 0, 0, nrAreas, 0, 0, "ignored")     
    is_indexed = pdb.gimp_drawable_is_indexed (QUANT_DRAWABLE) 
    if is_indexed == 1:
        hidden_message = "OK, converted"
    else:    
        erro_msg ("ERROR! QUANT_IMAGE not converted, exit")
        return
    #  
    # re-set image to RGB to proceed
    # ==============================
    pdb.gimp_image_convert_rgb (QUANT_IMAGE)
    pdb.gimp_layer_add_alpha (QUANT_DRAWABLE)                        #  RGBA
    
    # =====================================================================
    # bring the created quantized layer back to the output image
    # ---------------------------------------------------------------------
    #
    
    quantLayer = pdb.gimp_layer_new_from_drawable(QUANT_DRAWABLE, newImage)
    newImage.add_layer (quantLayer, 0)
    quantLayer.mode = NORMAL_MODE
    quantLayer.name = "quantLayer: Quantized Image Layer"
    quantLayer.opacity = 100.0
    # increase contrast
    pdb.gimp_brightness_contrast (quantLayer, 0, +50)
    # remove pure white
    pdb.gimp_curves_spline (quantLayer, HISTOGRAM_VALUE, 6, (0,0,232,232,255,232))
    # delete work imnages
    pdb.gimp_image_delete (QUANT_IMAGE)
    
    newsprintLayer = quantLayer.copy()
    newImage.add_layer (newsprintLayer, 0)
    newsprintLayer.mode = NORMAL_MODE
    newsprintLayer.name = "newsprintLayer: News-print Image Layer"
    newsprintLayer.opacity = 100.0 
    pdb.plug_in_newsprint (newImage, newsprintLayer,
                           5, #cell width
                           1, #color space = Separate to { GRAYSCALE (0), RGB (1), CMYK (2), LUMINANCE (3) }
                           0, #% black to pull out (CMYK only)
                           0,0, #gray angle, gray spot { DOTS (0), LINES (1), DIAMONDS (2), EUCLIDIAN-DOT (3), PS-DIAMONDS (4) }
                           5.0,1, #red angle, gray spot
                           5.0,1, #green angle, gray spot
                           5.0,1, #blue angle, gray spot
                           1)   #oversample
    
    shiftedHLayer = newsprintLayer.copy()
    newImage.add_layer (shiftedHLayer, 0)
    shiftedHLayer.mode = NORMAL_MODE
    shiftedHLayer.name = "shiftedHLayer: Shifted horizontally Image Layer"
    shiftedHLayer.opacity = 100.0 
    pdb.plug_in_shift (newImage, shiftedHLayer, 15, 1)
    pdb.plug_in_spread (newImage, shiftedHLayer, 1, 3)
    pdb.gimp_hue_saturation (shiftedHLayer, 0, 0, 0, +30)
    
    shiftedVLayer = newsprintLayer.copy()
    newImage.add_layer (shiftedVLayer, 0)
    shiftedVLayer.mode = GRAIN_MERGE_MODE
    shiftedVLayer.name = "shiftedVLayer: Shifted vertically Image Layer"
    shiftedVLayer.opacity = 60.0 
    pdb.plug_in_shift (newImage, shiftedVLayer, 45, 1)
    pdb.plug_in_spread (newImage, shiftedVLayer, 2, 1)
    pdb.gimp_hue_saturation (shiftedVLayer, 0, 0, 0, +30)
    
    knitpatternLayer = pdb.gimp_layer_new (newImage, newwidth, newheight, RGBA_IMAGE,
                                           "knitpatternLayer: Knit Pattern", 85.0, MULTIPLY_MODE)
    newImage.add_layer (knitpatternLayer, 0)
    pdb.gimp_context_set_pattern ("MShine07.png")
    pdb.gimp_edit_fill (knitpatternLayer, PATTERN_FILL)
    pdb.gimp_desaturate (knitpatternLayer)
    pdb.gimp_brightness_contrast (knitpatternLayer, +50, +70)
    pdb.plug_in_ripple (newImage, knitpatternLayer, 20, 6, 1, 0, 1, 0, 0)
    
    #pdb.gimp_image_raise_item_to_top (newImage, newBG)
    topBG = newBG.copy()    # background layer
    topBG.name = "topBG: Background Image Layer"
    newImage.add_layer (topBG, 0)
    # adjust values
    pdb.gimp_curves_spline (topBG, HISTOGRAM_VALUE, 8, (0,0,48,96,207,159,255,255))
    topBG.mode = HARDLIGHT_MODE     
    topBG.opacity = 50.0
    
    ######################################################################################################################
    
    if (doFlatten == True) :
        pdb.gimp_item_set_visible (newBG, False)
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_item_set_visible (newBG, True)
    else:
        Outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (Outcome, 0)
        
    Outcome.name = st+" "+name+suffix    
    
    pdb.gimp_display_new(newImage)
    
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_image_undo_group_end(inImage)
    
    pdb.gimp_context_pop()
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    return(Outcome)


# This is the plugin registration function
register(
    "PlainKnittedLook",    
    "to create a new image from the source drawable having a plain knitted look",   
    "This script creates a new image from the source drawable having a plain knitted look.",
    "Diego", 
    "Diego Nassetti ", 
    "October 2017",
    "PlainKnittedLook", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+
                                "\n (0=same as input)"+
                                "\n (not smaller than 2000)", 2400, (0, 6000, 200)),
      (PF_ADJUSTMENT, "colorareas", "How many colour areas"+"\n (0=auto)"+
                              "\n (8-32 recommended)", 0, (0, 80, 1)),
      (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),
    ],
    PlainKnittedLook,
    menu="<Image>/Diego/Artistic",
    )

main()
